# Generated by roxygen2: do not edit by hand

S3method(format_corr,default)
S3method(format_corr,vcmat_ar1)
S3method(format_corr,vcmat_cs)
S3method(format_corr,vcmat_diag)
S3method(format_corr,vcmat_hetar1)
S3method(format_corr,vcmat_homcs)
S3method(format_corr,vcmat_homdiag)
S3method(get_sd,default)
S3method(get_sd,vcmat_ar1)
S3method(get_sd,vcmat_homcs)
S3method(get_sd,vcmat_homdiag)
export("RHSForm<-")
export(RHSForm)
export(addForm)
export(addForm0)
export(anySpecial)
export(drop.special)
export(dropHead)
export(expandAllGrpVar)
export(expandDoubleVert)
export(expandDoubleVerts)
export(expandGrpVar)
export(extractForm)
export(findbars)
export(findbars_x)
export(formatVC)
export(format_corr)
export(format_sdvar)
export(get_sd)
export(inForm)
export(isNested)
export(makeOp)
export(mkReTrms)
export(noSpecials)
export(no_specials)
export(nobars)
export(nobars_)
export(randint)
export(reOnly)
export(replaceForm)
export(splitForm)
export(sub_specials)
export(subbars)
export(sumTerms)
importFrom(Matrix,KhatriRao)
importFrom(Matrix,drop0)
importFrom(Matrix,fac2sparse)
importFrom(Matrix,sparse.model.matrix)
importFrom(Matrix,sparseMatrix)
importFrom(Rdpack,reprompt)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,as.formula)
importFrom(stats,formula)
importFrom(stats,model.matrix)
importFrom(stats,reformulate)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(utils,head)
importMethodsFrom(Matrix,coerce)
importMethodsFrom(Matrix,diag)
importMethodsFrom(Matrix,t)
