% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{expandDoubleVerts}
\alias{expandDoubleVerts}
\title{Expand terms with \code{'||'} notation into separate \code{'|'} terms}
\usage{
expandDoubleVerts(term)
}
\arguments{
\item{term}{a mixed-model formula}
}
\value{
the modified term
}
\description{
From the right hand side of a formula for a mixed-effects model,
expand terms with the double vertical bar operator
into separate, independent random effect terms.
}
\seealso{
\code{\link{formula}}, \code{\link{model.frame}}, \code{\link{model.matrix}}.

Other utilities: 
\code{\link{mkReTrms}()},
\code{\link{nobars}()},
\code{\link{subbars}()}
}
\concept{utilities}
\keyword{models}
\keyword{utilities}
